#extension GL_OES_EGL_image_external : require

precision mediump float;

varying mediump vec2 vTextureCoord;
uniform samplerExternalOES sTexture;

uniform float uOutputWidth;
uniform float uOutputHeight;
uniform float uLevel;
uniform float uBrightnessOffset;

const float W1 = 0.3 / 8.0;
const float W2 = 0.3 / 16.0;

float lum(vec3 color) {
  return dot(color, vec3(0.2126, 0.7152, 0.0722));
}

float blur(vec2 step) {
    float blur = lum(texture2D(sTexture, vTextureCoord).rgb) * (0.4 - W1);
    for (int i = 0; i < 5; i++) {
       for (int j = 0; j < 5; j++) {
         vec2 coord = vTextureCoord.xy + step * vec2(i - 2, j - 2);
         float weight = (i == 0 || i == 4 || j == 0 || j == 4) ? W2 : W1;
         blur += lum(texture2D(sTexture, coord).rgb) * weight;
       }
    }

    return blur;
}

float overlayBlend(float v) {
    if (v < 0.5) {
        for (int i = 0; i < 4; i++) {
            v = v * v * 2.0;
        }
    } else {
        for (int i = 0; i < 4; i++) {
            v = 1.0 - ((1.0 - v) * (1.0 - v) * 2.0);
        }
    }

    return v;
}

void main() {
    vec3 origin = texture2D(sTexture, vTextureCoord).rgb;
    vec2 step = 8.0 / vec2(uOutputWidth, uOutputHeight);

    float highPass = overlayBlend(lum(origin) - blur(step) + 0.5);
    float alpha = pow(lum(origin), 0.5 / uLevel);

    vec3 color = origin - (vec3(highPass) - origin) * alpha * 0.09;
    gl_FragColor = vec4(mix(color, max(origin, color), alpha) - uBrightnessOffset, 1.0);
}
